#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

struct pendu_s {
    char* mot;
    int* lettres_trouvees;
    int len_mot;
    int nb_essais;
    int nb_lettres_trouvees;
};


int pendu_tester_lettre(struct pendu_s* pendu, char lettre)
{
    int nb = 0;

    for (int i = 0; i < pendu->len_mot; i++)
    {
        if (pendu->mot[i] == lettre)
        {
            if (pendu->lettres_trouvees[i])
            {
                return -1;
            }

            pendu->lettres_trouvees[i] = 1;
            nb++;
        }
    }

    pendu->nb_lettres_trouvees += nb;
    pendu->nb_essais++;

    return nb;
}

struct pendu_s* init_pendu(char* mot)
{
    struct pendu_s* pendu = malloc(sizeof(struct pendu_s));
    if (pendu == NULL)
    {
        return NULL;
    }

    pendu->mot = mot;
    pendu->len_mot = strlen(mot);
    pendu->nb_essais = 0;
    pendu->nb_lettres_trouvees = 0;
    pendu->lettres_trouvees = calloc(pendu->len_mot, sizeof(int));
    if (pendu->lettres_trouvees == NULL)
    {
        free(pendu);
        return NULL;
    }

    return pendu;
}

int pendu_gagne(struct pendu_s* pendu)
{
    return pendu->nb_lettres_trouvees == pendu->len_mot;
}

void pendu_affiche_mot(struct pendu_s* pendu)
{
    for (int i = 0; i < pendu->len_mot; i++)
    {
        if (pendu->lettres_trouvees[i])
        {
            printf("%c", pendu->mot[i]);
        }
        else
        {
            printf("-");
        }
    }
}

void pendu_libere(struct pendu_s* pendu)
{
    free(pendu->lettres_trouvees);
    free(pendu);
}

void pendu_affiche_victoire(struct pendu_s* pendu)
{
    assert(pendu_gagne(pendu));

    printf("Bravo ! Vous avez trouvé le mot en %d coups ! Le mot était '%s'.\n", pendu->nb_essais, pendu->mot);
}

int main(int argc, char* argv[])
{
    if (argc != 2)
    {
        printf("Usage: %s <mot à trouver>\n", argv[0]);
        return EXIT_FAILURE;
    }

    struct pendu_s* pendu = init_pendu(argv[1]);
    if (pendu == NULL)
    {
        printf("Erreur lors de l'allocation mémoire\n");
        return EXIT_FAILURE;
    }

    printf("\e[1;1H\e[2J");

    while (!pendu_gagne(pendu))
    {
        printf("Il faut trouver: ");
        pendu_affiche_mot(pendu);
        printf("\n");

        printf("Proposez une lettre : ");

        char lettre;
        scanf(" %1c", &lettre);

        int nb_trouvees = pendu_tester_lettre(pendu, lettre);
        if (nb_trouvees == 0)
        {
            printf("Il n'y a pas de lettre '%c' dans le mot à trouver...\n", lettre);
        }
        else if (nb_trouvees == -1)
        {
            printf("La lettre '%c' a déjà été trouvée...\n", lettre);
        }
        else
        {
            printf("Oui ! La lettre '%c' est présente %d fois !\n", lettre, nb_trouvees);
        }
    }

    pendu_affiche_victoire(pendu);

    pendu_libere(pendu);

    return EXIT_SUCCESS;
}
